#
# Copyright 2017 Centreon (http://www.centreon.com/)
#
# Centreon is a full-fledged industry-strength solution that meets
# the needs in IT infrastructure and application monitoring for
# service performance.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

package hardware::server::dell::openmanage::snmp::mode::hardware;

use base qw(centreon::plugins::mode);

use strict;
use warnings;
use centreon::plugins::misc;

my $thresholds = {
    globalstatus => [
        ['other', 'CRITICAL'],
        ['unknown', 'UNKNOWN'],
        ['ok', 'OK'],
        ['nonCritical', 'WARNING'],
        ['critical', 'CRITICAL'],
        ['nonRecoverable', 'CRITICAL'],
    ],
    battery => [
        ['other', 'CRITICAL'],
        ['unknown', 'UNKNOWN'],
        ['ok', 'OK'],
        ['nonCritical', 'WARNING'],
        ['critical', 'CRITICAL'],
        ['nonRecoverable', 'CRITICAL'],
    ],
    cachebattery => [
        ['other', 'CRITICAL'],
        ['unknown', 'UNKNOWN'],
        ['ok', 'OK'],
        ['nonCritical', 'WARNING'],
        ['critical', 'CRITICAL'],
        ['nonRecoverable', 'CRITICAL'],
    ],
    connector => [
        ['other', 'CRITICAL'],
        ['unknown', 'UNKNOWN'],
        ['ok', 'OK'],
        ['nonCritical', 'WARNING'],
        ['critical', 'CRITICAL'],
        ['nonRecoverable', 'CRITICAL'],
    ],
    controller => [
        ['other', 'CRITICAL'],
        ['unknown', 'UNKNOWN'],
        ['ok', 'OK'],
        ['nonCritical', 'WARNING'],
        ['critical', 'CRITICAL'],
        ['nonRecoverable', 'CRITICAL'],
    ],
    controller => [
        ['other', 'CRITICAL'],
        ['unknown', 'UNKNOWN'],
        ['ok', 'OK'],
        ['nonCritical', 'WARNING'],
        ['critical', 'CRITICAL'],
        ['nonRecoverable', 'CRITICAL'],
    ],
    cpu => [
        ['other', 'CRITICAL'],
        ['unknown', 'UNKNOWN'],
        ['ok', 'OK'],
        ['nonCritical', 'WARNING'],
        ['critical', 'CRITICAL'],
        ['nonRecoverable', 'CRITICAL'],
    ],
    esmlog => [
        ['other', 'CRITICAL'],
        ['unknown', 'UNKNOWN'],
        ['ok', 'OK'],
        ['nonCritical', 'WARNING'],
        ['critical', 'CRITICAL'],
        ['nonRecoverable', 'CRITICAL'],
    ],
    fan => [
        ['other', 'CRITICAL'], 
        ['unknown', 'UNKNOWN'], 
        ['ok', 'OK'], 
        ['nonCriticalUpper', 'WARNING'],
        ['criticalUpper', 'CRITICAL'],
        ['nonRecoverableUpper', 'CRITICAL'],
        ['nonCriticalLower', 'WARNING'],
        ['criticalLower', 'CRITICAL'],
        ['nonRecoverableLower', 'CRITICAL'],
        ['failed', 'CRITICAL']
    ],
    logicaldrive => [
        ['other', 'CRITICAL'],
        ['unknown', 'UNKNOWN'],
        ['ok', 'OK'],
        ['nonCritical', 'WARNING'],
        ['critical', 'CRITICAL'],
        ['nonRecoverable', 'CRITICAL'],
    ],
    memory => [
        ['other', 'CRITICAL'],
        ['unknown', 'UNKNOWN'],
        ['ok', 'OK'],
        ['nonCritical', 'WARNING'],
        ['critical', 'CRITICAL'],
        ['nonRecoverable', 'CRITICAL'],
    ],
    physicaldisk => [
        ['other', 'CRITICAL'],
        ['unknown', 'UNKNOWN'],
        ['ok', 'OK'],
        ['nonCritical', 'WARNING'],
        ['critical', 'CRITICAL'],
        ['nonRecoverable', 'CRITICAL'],
    ],
    physicaldisk_smartalert => [
        ['yes', 'WARNING'],
        ['no', 'OK'],
    ],
    psu => [
        ['other', 'CRITICAL'],
        ['unknown', 'UNKNOWN'],
        ['ok', 'OK'],
        ['nonCritical', 'WARNING'],
        ['critical', 'CRITICAL'],
        ['nonRecoverable', 'CRITICAL'],
    ],
    temperature => [
        ['other', 'CRITICAL'], 
        ['unknown', 'UNKNOWN'], 
        ['ok', 'OK'], 
        ['nonCriticalUpper', 'WARNING'],
        ['criticalUpper', 'CRITICAL'],
        ['nonRecoverableUpper', 'CRITICAL'],
        ['nonCriticalLower', 'WARNING'],
        ['criticalLower', 'CRITICAL'],
        ['nonRecoverableLower', 'CRITICAL'],
        ['failed', 'CRITICAL'],  
    ],
};

sub new {
    my ($class, %options) = @_;
    my $self = $class->SUPER::new(package => __PACKAGE__, %options);
    bless $self, $class;
    
    $self->{version} = '1.0';
    $options{options}->add_options(arguments =>
                                { 
                                  "exclude:s"        => { name => 'exclude' },
                                  "component:s"      => { name => 'component', default => '.*' },
                                  "no-component:s"   => { name => 'no_component' },
                                  "threshold-overload:s@"   => { name => 'threshold_overload' },
                                  "warning:s@"              => { name => 'warning' },
                                  "critical:s@"             => { name => 'critical' },
                                });

    $self->{components} = {};
    return $self;
}

sub check_options {
    my ($self, %options) = @_;
    $self->SUPER::init(%options);
    
    if (defined($self->{option_results}->{no_component})) {
        if ($self->{option_results}->{no_component} ne '') {
            $self->{no_components} = $self->{option_results}->{no_component};
        } else {
            $self->{no_components} = 'critical';
        }
    }
    
    $self->{overload_th} = {};
    foreach my $val (@{$self->{option_results}->{threshold_overload}}) {
        if ($val !~ /^(.*?),(.*?),(.*)$/) {
            $self->{output}->add_option_msg(short_msg => "Wrong threshold-overload option '" . $val . "'.");
            $self->{output}->option_exit();
        }
        my ($section, $status, $filter) = ($1, $2, $3);
        if ($self->{output}->is_litteral_status(status => $status) == 0) {
            $self->{output}->add_option_msg(short_msg => "Wrong threshold-overload status '" . $val . "'.");
            $self->{output}->option_exit();
        }
        $self->{overload_th}->{$section} = [] if (!defined($self->{overload_th}->{$section}));
        push @{$self->{overload_th}->{$section}}, {filter => $filter, status => $status};
    }
    
    $self->{numeric_threshold} = {};
    foreach my $option (('warning', 'critical')) {
        foreach my $val (@{$self->{option_results}->{$option}}) {
            if ($val !~ /^(.*?),(.*?),(.*)$/) {
                $self->{output}->add_option_msg(short_msg => "Wrong $option option '" . $val . "'.");
                $self->{output}->option_exit();
            }
            my ($section, $regexp, $value) = ($1, $2, $3);
            if ($section !~ /^(temperature|fan|psu\.power)$/) {
                $self->{output}->add_option_msg(short_msg => "Wrong $option option '" . $val . "' (type must be: temperature, fan, psu.power).");
                $self->{output}->option_exit();
            }
            my $position = 0;
            if (defined($self->{numeric_threshold}->{$section})) {
                $position = scalar(@{$self->{numeric_threshold}->{$section}});
            }
            if (($self->{perfdata}->threshold_validate(label => $option . '-' . $section . '-' . $position, value => $value)) == 0) {
                $self->{output}->add_option_msg(short_msg => "Wrong $option threshold '" . $value . "'.");
                $self->{output}->option_exit();
            }
            $self->{numeric_threshold}->{$section} = [] if (!defined($self->{numeric_threshold}->{$section}));
            push @{$self->{numeric_threshold}->{$section}}, { label => $option . '-' . $section . '-' . $position, threshold => $option, regexp => $regexp };
        }
    }
}

sub run {
    my ($self, %options) = @_;
    $self->{snmp} = $options{snmp};
    
    # In '10892-MIB'
    my $oid_chassisModelName = ".1.3.6.1.4.1.674.10892.1.300.10.1.9";
    my $snmp_request = [ { oid => $oid_chassisModelName } ];
    
    my @components = ('globalstatus', 'fan', 'psu', 'temperature', 'cpu', 'cachebattery', 'memory',
                      'physicaldisk', 'logicaldrive', 'esmlog', 'battery', 'controller', 'connector');
    foreach (@components) {
        if (/$self->{option_results}->{component}/) {
            my $mod_name = "hardware::server::dell::openmanage::snmp::mode::components::$_";
            centreon::plugins::misc::mymodule_load(output => $self->{output}, module => $mod_name,
                                                   error_msg => "Cannot load module '$mod_name'.");
            my $func = $mod_name->can('load');
            $func->(request => $snmp_request); 
        }
    }
    
    if (scalar(@{$snmp_request}) == 1) {
        $self->{output}->add_option_msg(short_msg => "Wrong option. Cannot find component '" . $self->{option_results}->{component} . "'.");
        $self->{output}->option_exit();
    }
    $self->{results} = $self->{snmp}->get_multiple_table(oids => $snmp_request);
    
    foreach my $oid ($self->{snmp}->oid_lex_sort(keys %{$self->{results}->{$oid_chassisModelName}})) {
        my $name = defined($self->{results}->{$oid_chassisModelName}->{$oid}) ? 
        centreon::plugins::misc::trim($self->{results}->{$oid_chassisModelName}->{$oid}) : 'unknown';
        $self->{output}->output_add(long_msg => sprintf("Product Name: %s", $name));
    }    
    
    foreach (@components) {
        if (/$self->{option_results}->{component}/) {
            my $mod_name = "hardware::server::dell::openmanage::snmp::mode::components::$_";
            my $func = $mod_name->can('check');
            $func->($self); 
        }
    }
    
    my $total_components = 0;
    my $display_by_component = '';
    my $display_by_component_append = '';
    foreach my $comp (sort(keys %{$self->{components}})) {
        # Skipping short msg when no components
        next if ($self->{components}->{$comp}->{total} == 0 && $self->{components}->{$comp}->{skip} == 0);
        $total_components += $self->{components}->{$comp}->{total} + $self->{components}->{$comp}->{skip};
        my $count_by_components = $self->{components}->{$comp}->{total} + $self->{components}->{$comp}->{skip}; 
        $display_by_component .= $display_by_component_append . $self->{components}->{$comp}->{total} . '/' . $count_by_components . ' ' . $self->{components}->{$comp}->{name};
        $display_by_component_append = ', ';
    }
    
    $self->{output}->output_add(severity => 'OK',
                                short_msg => sprintf("All %s components are ok [%s].", 
                                                     $total_components,
                                                     $display_by_component)
                                );

    if (defined($self->{option_results}->{no_component}) && $total_components == 0) {
        $self->{output}->output_add(severity => $self->{no_components},
                                    short_msg => 'No components are checked.');
    }

    $self->{output}->display();
    $self->{output}->exit();
}

sub check_exclude {
    my ($self, %options) = @_;

    if (defined($options{instance})) {
        if (defined($self->{option_results}->{exclude}) && $self->{option_results}->{exclude} =~ /(^|\s|,)${options{section}}[^,]*#\Q$options{instance}\E#/) {
            $self->{components}->{$options{section}}->{skip}++;
            $self->{output}->output_add(long_msg => sprintf("Skipping $options{section} section $options{instance} instance."));
            return 1;
        }
    } elsif (defined($self->{option_results}->{exclude}) && $self->{option_results}->{exclude} =~ /(^|\s|,)$options{section}(\s|,|$)/) {
        $self->{output}->output_add(long_msg => sprintf("Skipping $options{section} section."));
        return 1;
    }
    return 0;
}

sub get_severity_numeric {
    my ($self, %options) = @_;
    my $status = 'OK'; # default
    my $thresholds = { warning => undef, critical => undef };
    my $checked = 0;
    
    if (defined($self->{numeric_threshold}->{$options{section}})) {
        my $exits = [];
        foreach (@{$self->{numeric_threshold}->{$options{section}}}) {
            if ($options{instance} =~ /$_->{regexp}/) {
                push @{$exits}, $self->{perfdata}->threshold_check(value => $options{value}, threshold => [ { label => $_->{label}, exit_litteral => $_->{threshold} } ]);
                $thresholds->{$_->{threshold}} = $self->{perfdata}->get_perfdata_for_output(label => $_->{label});
                $checked = 1;
            }
        }
        $status = $self->{output}->get_most_critical(status => $exits) if (scalar(@{$exits}) > 0);
    }
    
    return ($status, $thresholds->{warning}, $thresholds->{critical}, $checked);
}

sub get_severity {
    my ($self, %options) = @_;
    my $status = 'UNKNOWN'; # default 
    
    if (defined($self->{overload_th}->{$options{section}})) {
        foreach (@{$self->{overload_th}->{$options{section}}}) {            
            if ($options{value} =~ /$_->{filter}/i) {
                $status = $_->{status};
                return $status;
            }
        }
    }
    foreach (@{$thresholds->{$options{section}}}) {           
        if ($options{value} =~ /$$_[0]/i) {
            $status = $$_[1];
            return $status;
        }
    }
    
    return $status;
}
1;

__END__

=head1 MODE

Check Hardware (Global status, Fans, CPUs, Power Supplies, Temperature, Storage).

=over 8

=item B<--component>

Which component to check (Default: '.*').
Can be: 'globalstatus', 'fan', 'cpu', 'psu', 'temperature', 'cachebattery',
'physicaldisk', 'logicaldrive', 'battery', 'controller', 'connector'.

=item B<--exclude>

Exclude some parts (comma seperated list) (Example: --exclude=fan,cpu)
Can also exclude specific instance: --exclude=fan#1#,cpu

=item B<--no-component>

Return an error if no compenents are checked.
If total (with skipped) is 0. (Default: 'critical' returns).

=item B<--threshold-overload>

Set to overload default threshold values (syntax: section,status,regexp)
It used before default thresholds (order stays).
Example: --threshold-overload='temperature,CRITICAL,^(?!(ok)$)'

=item B<--warning>

Set warning threshold for temperature, psu.power or fan (syntax: type,regexp,threshold)
Example: --warning='temperature,.*,30'

=item B<--critical>

Set critical threshold for temperature, psu.power or fan (syntax: type,regexp,threshold)
Example: --critical='temperature,.*,40'

=back

=cut
    
